# An Anonymous Brainstorming Tool
# See the application description at the bottom of this file
# for details

# Initialize conference
gk_initConf $argv

# Tell the window manager the title and the default window size
wm title . "Brainstorming Tool"
wm minsize . 20 7

# Create the default groupkit pulldown menu bar 
# Then add Clear, Open, and Save items to the "File" menu 
gk_defaultMenu .menubar
.menubar.file.menu add separator
.menubar.file.menu add command -label "Open" -command "Open"
.menubar.file.menu add command -label "Save" -command "Save"
.menubar.file.menu add separator
.menubar.file.menu add command -label "Clear Ideas" -command "Clear"
pack .menubar -side top -fill x 

# Make a frame in the window's middle which has a scrollable listbox
# containing the shared ideas. 
# The twist is that the scrollbar is actually the Groupkit multiuser scrollbar!
frame .middle -bd 2 -relief raised
listbox .middle.shared_list \
    -setgrid true \
    -yscrollcommand "gk_scroller .middle.vscroll" 
gk_groupScroll .middle.vscroll -command ".middle.shared_list yview"
pack .middle -side top  -expand yes -fill both
pack .middle.vscroll -side right -fill y
pack .middle.shared_list -side left -expand yes -fill both

# Make a frame at the bottom that contains a labelled entry where
# the user will type their idea.
frame .bottom
label .bottom.label \
    -text " Enter Idea:" 
entry .bottom.idea_entry \
    -textvariable myIdea \
    -relief sunken 
focus .bottom.idea_entry

pack .bottom -side bottom -fill x
pack .bottom.label -side left 
pack .bottom.idea_entry -side left -expand yes -fill x

# After a carriage return, the idea is entered into the shared list
bind .bottom.idea_entry <Return> "insert_idea_into_shared_list"

# Tell all instances of this application to insert the idea (if it 
# isn't an empty string) into the shared list.
proc insert_idea_into_shared_list {} { global myIdea
    if {$myIdea != ""} {
        gk_toAll actually_insert_idea_into_shared_list $myIdea
        set myIdea ""
    }
}

# Actually insert the idea into the shared list
proc actually_insert_idea_into_shared_list {idea} {
    .middle.shared_list insert end $idea
}

# ================================================================
# Update the entering participant with the state of the conference 
# The trace on gk_entrantUserNum is only invoked for ONE of the 
# existing users when a new user joins the conference.
# updateEntrant puts all the existing ideas into the new user's listbox.
trace variable gk_entrantUserNum w {updateEntrant}
proc updateEntrant args { global gk_entrantUserNum _gk_localUser 
    for {set count 0} {$count < [.middle.shared_list size]} {incr count 1} {
	set idea [.middle.shared_list get $count]
        gk_toUserNum $gk_entrantUserNum \
	    actually_insert_idea_into_shared_list $idea 
    }
}
# ================================================================
# These routines are invoked by the commands in the "File" menu.
# They save the ideas in a file, or open a new set of 
# previously saved ideas, or clears the ideas in all participant's idea lists.

# Saves contents of listbox in file.  The FSBox routine asks the user which
# file to save into, the file is opened and the listbox elements are put in.
# If the file already exists, the data is appendend onto it.
proc Save {} { 
    if {[.middle.shared_list size] == 0} {return}
    set fileName [FSBox]
    if {$fileName == ""} {return}
    set fd [open $fileName a]
    for {set count 0} {$count < [.middle.shared_list size]} {incr count 1} {
	puts $fd [.middle.shared_list get $count]
    }
    close $fd
}

# Gets contents of a file and displays them, line by line, in the
# listboxes of ALL the conference processes.
proc Open {} {
    set fileName [FSBox]
    if {$fileName == ""} {return}
    Clear
    set fd [open $fileName r]
    set result [gets $fd temp]
    while {$result != -1} {
	gk_toAll actually_insert_idea_into_shared_list $temp
	set result [gets $fd temp]
    }
    close $fd
}    

# Clears ALL the listboxes.
proc Clear {} { 
    gk_toAll doClear 
}

# Clear a listbox.
proc doClear {} { 
    .middle.shared_list delete 0 end
}

# ================================================================
# These variables/routines are used to describe this application
# to the user. An "About this application" item is added to the
# Help menu, which will pop up an information window

set application(title) \
    "Brainstorming Tool"

set application(description) \
"You can enter one-line 'ideas' by typing text followed by <return>. 

Ideas of all participants,\
including yours, are displayed in the scrollable list.

Limitations:
-Control characters are handled poorly."

.menubar.help.menu add command \
    -label "About This Application" \
    -command {about_this_applicaton}


# Create a window that describes this application to the user
proc about_this_applicaton {} {global application
    if {[winfo exists .about_window] == 1} { return }
    toplevel .about_window 
    wm title .about_window "About this application"
    label .about_window.title \
	-text $application(title) \
	-font -Adobe-times-bold-r-normal--*-180*
    message .about_window.text  -width 240 \
	-text $application(description)
    button .about_window.ok -text "Okay" -command "destroy .about_window"
    pack .about_window.title .about_window.text .about_window.ok -side top
}

