#
# Command Line Registrar Client
# -----------------------------
#
# This provides a very light-weight command line Registrar Client (see rc.tcl
# for information on Registrar Clients in general).  The idea with this client
# is to allow joining "standard" conferences very easily, such as a the
# PostIt conference for example (see postit.tcl).  As such, this Registrar
# Client could be started up in a .xinitrc file.
#
# This registrar client is started up indirectly via a one line shell
# script (see cmdrc).  A typical invocation would be:
#
#      cmdrc PostIt
#
# This will join a conference named "PostIt" if it already exists, or create
# the conference if it doesn't yet exist.  The conference will be created
# with the type "PostIt" (which indirectly specifies the groupware conference
# application to be used; see the documentation for the proc _gk_createConference
# in rc.tcl for details). 
#
# This Registrar Client remains running until the conference that it spawns
# terminates.
#

_gk_initGkErrorHandling

#
# Maintain a flag specifying if we've initiated a join to the conference.
# The global target_name is the name of the conference we'd like to join,
#   e.g. PostIt
#

set already_joined 0
set target_name [lindex $argv 2]
wm withdraw .

_gk_pollConferences


#
# This is called by _gk_updateList when a new conference is found.  If the
# conference is the one we're looking for, join to it.  Update our conference
# list.
#

proc gk_foundNewConf conf {    global _gk_rcConfsList _gk_rcUsersList already_joined target_name
    _gk_pollUsers [keylget conf confnum]
    lappend _gk_rcConfsList $conf
    set _gk_rcUsersList([keylget conf confnum]) {}
    if {([keylget conf confname] == $target_name) && ($already_joined ==0)} {
	set already_joined 1
	_gk_callJoinConference [keylget conf confnum]
	_gk_pollUsers [keylget conf confnum]
    }
}


#
# This is called bu _gk_updateList when a new user is found.  If the new user
# is us, create the conference.  Update our user list.
#

proc gk_foundNewUser user {    global host myport _gk_rcUsersList
    set confnum [keylget user confnum]
    lappend _gk_rcUsersList($confnum) $user

    if {([keylget user host] == $host) && ([keylget user port] == $myport)} {
	_gk_createConference [_gk_findConfInfo $confnum] [keylget user usernum]
	 foreach i $_gk_rcUsersList($confnum) {
            if {[keylget i usernum] < [keylget user usernum]} {
                _gk_joinTo $i $user
            }
        }	

    }
}


#
# This is called by _gk_updateList when a deleted user is found.  Update our
# user list.  Pass the delete on to our conference.
#

proc gk_foundDeletedUser {conf user} {  global _gk_rcUsersList
    set idx 0; foreach i $_gk_rcUsersList($conf) {
	if {[keylget i usernum] == $user} {
	    _gk_toConf $conf "_gk_removeUser [list [lindex $_gk_rcUsersList($conf) $idx]]"
	    set _gk_rcUsersList($conf) [lreplace $_gk_rcUsersList($conf) $idx $idx]
	} else { incr idx }
    }
}


#
# This is called by _gk_updateList when a deleted conference is found.  Update
# our conference list.  Pass the delete on to our conference.
#

proc gk_foundDeletedConf conf { global _gk_rcConfsList _gk_rcUsersList
    set idx 0; foreach i $_gk_rcConfsList {
	if {[keylget i confnum] == $conf} {
	    set _gk_rcConfsList [lreplace $_gk_rcConfsList $idx $idx]
	    set _gk_rcUsersList($conf) ""
	    _gk_toConf $conf "_gk_deleteConf"
	} else { incr idx }
    }
}


#
# This overrides the _gk_confList routine in rc.tcl.  Basically after going
# through the entire list of conferences, we check to see if we've found
# the conference we're looking for.  If not, it must not be there so we
# create the conference ourself.
#

proc _gk_confList _gk_regConfsList {  global already_joined host myport target_name registrar
    _gk_updateList $_gk_regConfsList _gk_rcConfsList _gk_rcConfsList confnum gk_foundNewConf gk_foundDeletedConf
    if {$already_joined == 0} {
	keylset conf confname $target_name conftype $target_name \
	    originator $host$myport
	_gk_callNewConference $conf
	after 1000 _gk_pollConferences
    }
}


#
# This is called by errors.tcl if a socket connection dies.  If it is the
# conference that we spawned dying, first pass this on to the central
# Registrar and then kill ourself.
#

proc socket_closed filedesc { global conferences
    foreach i $conferences {
	if {[keylget i filedesc] == $filedesc} {
	    _gk_userLeft [keylget i confnum] [keylget i usernum]
	    after 1000 destroy .
	}
    }
}



